@@ -277,7 +277,9 @@
    ((some-fn number? odd? #(> % 0)) 2 4 6 8 -10)
    ;; 3 preds, short-circuiting
    ((some-fn number? odd? #(> % 0)) 1 :a)
+   ((some-fn number? odd? #(> % 0)) :a 1)
    ((some-fn number? odd? #(> % 0)) 1 3 :a)
+   ((some-fn number? odd? #(> % 0)) :a 1 3)
    ((some-fn number? odd? #(> % 0)) 1 3 5 :a)
    ((some-fn number? odd? #(> % 0)) 1 :a 3 5 7)
    ;; 4 preds
@@ -379,3 +381,21 @@
     ;; rest arity
     {:a 5} (update {:a 1} :a + 1 1 1 1)
     {:a 6} (update {:a 1} :a + 1 1 1 1 1)))
+
+(deftest test-update-vals
+  (let [inm  (with-meta {:a 1 :b 2} {:has :meta})]
+    (are [result expr] (= result expr)
+      {:a 2 :b 3}   (update-vals inm inc)
+      {:has :meta}  (meta (update-vals inm inc))
+      {0 2 2 4}     (update-vals (hash-map 0 1 2 3) inc)
+      {0 2 2 4}     (update-vals (array-map 0 1 2 3) inc)
+      {0 2 2 4}     (update-vals (sorted-map 2 3 0 1) inc))))
+
+(deftest test-update-keys
+  (let [inm  (with-meta {:a 1 :b 2} {:has :meta})]
+    (are [result expr] (= result expr)
+      {"a" 1 "b" 2} (update-keys inm name)
+      {:has :meta}  (meta (update-keys inm name))
+      {1 1 3 3}     (update-keys (hash-map 0 1 2 3) inc)
+      {1 1 3 3}     (update-keys (array-map 0 1 2 3) inc)
+      {1 1 3 3}     (update-keys (sorted-map 2 3 0 1) inc))))
