@@ -51,4 +51,13 @@
   (checkCLJ2066 (javax.xml.stream.XMLInputFactory/newInstance))
 
   ;; CLJ-2414 - find default method on interface of inaccessible class
-  (checkCLJ2414 (java.nio.file.Paths/get "src" (into-array String []))))
\ No newline at end of file
+  (checkCLJ2414 (java.nio.file.Paths/get "src" (into-array String []))))
+
+(defn sleep [ms]
+  ;; in Java <19, does not reflect. Java >= 19, does reflect
+  (Thread/sleep ms))
+
+
+(deftest check-CLJ-2843
+  (sleep 1)
+  (sleep (Integer/valueOf 1)))
